
/***** Program pro MCU ATmega8, zobrazovac jednotka, Ing. Marin Zl *****/

#include <avr/io.h>
#include <compat/deprecated.h>
#include <util/delay.h>

volatile char adresa = 0x00;
volatile char predchozi_adresa = 0x00;

char prijmi_znak(void)	// funkce pro pjem znaku z pijmae MRX1
{
	char znak = 0xFF;
	char znak_b3 = 0;
	char znak_b2 = 0;
	char znak_b1 = 0;
	char znak_b0 = 0;

	// naten a vymaskovn "adresy"
	adresa = ~PINC;
	adresa = (adresa & 0b00110000)>>4;

	if (adresa != 0x03 && adresa != 0x00)	// adresa (piny) jsou neekvivalentn
	{
		if(adresa != predchozi_adresa)		// adresa se nerovn t pedchoz tzn. pila nov
		{
			// ten znaku z portu C a jeho negace z dvodu negovanho vstupu pijmae
			znak = ~PINC;

			// rozklad promnn znak na jednotliv bity
			znak_b3 = (znak & 0b00001000) >> 3;
			znak_b2 = (znak & 0b00000100) >> 2;
			znak_b1 = (znak & 0b00000010) >> 1;
			znak_b0 = (znak & 0b00000001);

			znak = 0;
			
			// sloen BCD znaku do sprvnho poad v zvislosti na jednotlivch pinech pijmae
			znak = znak | (znak_b3 << 0) | (znak_b2 << 1) | (znak_b1 << 2) | (znak_b0 << 3);
		}
	}

	if (adresa != 0x03 && adresa != 0x00) { predchozi_adresa = adresa; }	// uloen pedchoz "adresy" pro porovnn
	
	return znak;
}

void zapis_displej(char pozice, char znak)	// funkce pro zpis znaku na velky zobrazovaci displej
{
	char znak_b3 = 0;
	char znak_b2 = 0;
	char znak_b1 = 0;
	char znak_b0 = 0;

	// rozklad promnn znak na jednotliv bity
	znak_b3 = (znak & 0b00001000) >> 3;
	znak_b2 = (znak & 0b00000100) >> 2;
	znak_b1 = (znak & 0b00000010) >> 1;
	znak_b0 = (znak & 0b00000001);

	// vymazn pedchozho zobrazenho znaku na displeji
	switch(pozice)
	{
		case 100: PORTD = PORTD & 0b00011111; PORTB = PORTB & 0b11111110; break;
		case 10:  PORTD = PORTD & 0b11100111; PORTB = PORTB & 0b00111111; break;
		case 1:   PORTD = PORTD & 0b11111000; PORTC = PORTC & 0b10111111; break;
		case 101: PORTB = PORTB & 0b11100001; break; 
	}

	// zapsn novho znaku bit po bitu, v zvislosti na jednotlivch pinech pipojenho aie displeje
	switch(pozice)
	{
		case 100: PORTD = PORTD | (znak_b3 << 6) | (znak_b2 << 7) |(znak_b0 << 5); 
				  PORTB = PORTB | (znak_b1 << 0); break;
		case 10:  PORTD = PORTD | (znak_b3 << 4) | (znak_b0 << 3); 
				  PORTB = PORTB | (znak_b2 << 6) | (znak_b1 << 7); break;
		case 1:   PORTD = PORTD | (znak_b3 << 0) | (znak_b2 << 1) |(znak_b1 << 2); 
				  PORTC = PORTC | (znak_b0 << 6); break;
		case 101: PORTB = PORTB | (znak_b3 << 3) | (znak_b2 << 2) | (znak_b1 << 1) | (znak_b0 << 4); break; 
	}


}

int main(void)
{
	char znak = 0xFF;
	char predchozi_znak = 0;
	char mod_desetina = 0;
	char tabulka_hodnot[3] = {0x0F,0x0F,0x0F};
	
	// nastaven vstupn - vstupnch pin
	PORTB=0xFF;
	DDRB=0xFF;	// vechny piny vstupn (na displeje)	
	PORTC=0x40;	
	DDRC=0x40;	// vechny vstupn krom PC6
	PORTD=0xFF;
	DDRD=0xFF;	// vechny piny vstupn (na displeje)

	cbi(PORTB,5); // zhasne desetinnou rku

	while(1)
    {	
		znak = prijmi_znak(); //teme znak z pijmae

		if (znak != predchozi_znak  && znak != 0xFF)	// nerovn se pedchozmu znaku a znak nen "przdn" tzn. byl zadn klavesnic
		{	
			// zpracovn a zpis na velk zobrazovac displej
			if (znak == 0x0F)	// zmknuto resetovac tlatko na klvesnici
			{
				// zpis przdn hodnoty na vechny pozici displeje
				zapis_displej(100, znak);
				zapis_displej(10, znak);
				zapis_displej(1, znak);
				zapis_displej(101, znak);

				cbi(PORTB,5);		// zhasne desetinnou rku
				mod_desetina = 0;	// zrume desetinn md

				// vymaeme tabulku hodnot
				tabulka_hodnot[2] = 0x0F;
				tabulka_hodnot[1] = 0x0F;
				tabulka_hodnot[0] = 0x0F;

			}
		
			else if (znak == 0x0E)	//zmknuto tlatko pro pesun na desetinnou pozici
			{
				sbi(PORTB,5);		// rozsvt desetinnou rku
				mod_desetina = 1;
			}
		
			else if (znak != 0xFF)	//zmknuto seln tlatko na klvesnici
			{
				if(mod_desetina == 0)	// nejsme v desetinnm mdu
				{
					// posun sla
					tabulka_hodnot[2] = tabulka_hodnot[1];	// druh znak se pesune na tet pozici
					tabulka_hodnot[1] = tabulka_hodnot[0];	// prvn znak se pesune druh pozici
					tabulka_hodnot[0] = znak;				// aktuln se pesune na prvn pozici

					// zpis sla na displej
					zapis_displej(100, tabulka_hodnot[2]);
					zapis_displej(10, tabulka_hodnot[1]);
					zapis_displej(1, tabulka_hodnot[0]);
				}

				if (mod_desetina == 1)	// jsme v desetinnm mdu
				{
					zapis_displej(101, znak);
				}
			}
	
		}
		
		predchozi_znak = znak;	// uloen pedchozho znaku pro porovnn

	}
	return 0;
}
